#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

#define PD_HOST "localhost"
#define PD_PORT 12345
#define PD_PATCH "/root/patches/simpleosc.pd"

int pd_sock;
pid_t pd_pid = 0;


/**
 * Forks off a Pd process and connects pd_sock to it
 */
void start_pd_and_connect(char *patch) {
	struct sockaddr_in sa;
	struct hostent *hp;
	int status;
	char *cmd[] = {"pd", "-nogui", "-noadc", "-nomidi", patch};
	
	// Fork off a Pd process
	pd_pid = fork();
	if(pd_pid == 0)
		execvp("pd", cmd);
	
	// Give Pd a little time to start up and open a socket
	usleep(500000);
	
	// Resolve hostname and connect
	if(! (hp = gethostbyname(PD_HOST))) {
		fprintf(stderr, "Can't resolve hostname %s\n", PD_HOST);
		exit(1);
	}
	
	pd_sock = socket(AF_INET, SOCK_STREAM, 0);
	if(pd_sock < 0) {
		fprintf(stderr, "Can't create socket\n");
		exit(1);
	}
	
	memcpy(&sa.sin_addr, hp->h_addr, hp->h_length);
	sa.sin_family = hp->h_addrtype;
	sa.sin_port = htons(PD_PORT);
	
	if((status = connect(pd_sock, (struct sockaddr*)&sa, sizeof(struct sockaddr_in))) < 0) {
		fprintf(stderr, "Couldn't connect.  Status: %d\n", status);
		exit(1);
	}
	printf("Connected to Pd\n");
}

/**
 * Closes the socket connected to Pd and terminates the Pd process
 */
void stop_pd_and_disconnect() {
	int i;
	close(pd_sock);
	
	if(pd_pid != 0) {
		printf("Stopping Pd\n");
		kill(pd_pid, SIGINT);
		waitpid(pd_pid, &i, 0);
	}
}


int main(int argc, char **argv) {
	int i, j;
	char buf[1024];
	start_pd_and_connect(PD_PATCH);
	
	write(pd_sock, "start;\n", 6);
	
	for(i=300; i<=600; i+= 100) {
		sprintf(buf, "freq %d;\n", i);
		write(pd_sock, buf, strlen(buf));
		
		for(j=0; j<=10; j+=2) {
			sprintf(buf, "amp %d;\n", j);
			write(pd_sock, buf, strlen(buf));
			sleep(1);
		}
	}
	
	stop_pd_and_disconnect();
	
	return 0;
}
